package com.tanpu.community.controller;


import com.tanpu.common.api.CommonResp;
import com.tanpu.community.api.beans.ForwardThemeReq;
import com.tanpu.community.api.beans.qo.MainTextQo;
import com.tanpu.community.api.beans.qo.ThemeQo;
import com.tanpu.community.api.beans.req.CreateCommentReq;
import com.tanpu.community.api.beans.req.CreateThemeReq;
import com.tanpu.community.manager.ThemeManager;
import com.tanpu.community.service.FansRelService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@Slf4j
@RequestMapping(value = "/api/theme")
public class ThemeController {

    @Autowired
    private ThemeManager themeManager;

    @Autowired
    private FansRelService fansRelService;

    @ApiOperation("发表主题")
    @PostMapping(value = "/publish")
    @ResponseBody
    public CommonResp publishTheme(@RequestBody CreateThemeReq req) {
        String userId = "liujm";
        themeManager.publishTheme(req, userId);
        return CommonResp.success();
    }

    @ApiOperation("圈子首页-推荐")
    @RequestMapping(value = "/recmend_list")
    @ResponseBody
    public List<ThemeQo> selectHotList() {
        String userId = "liujm";
        return themeManager.selectHotThemes(userId);
    }

    @ApiOperation("圈子首页-关注")
    @RequestMapping(value = "/interest_list")
    @ResponseBody
    public List<ThemeQo> selectInterestList() {
        String userId = "liujm012";
        return themeManager.selectInterestThemes(userId);
    }

    @ApiOperation("评论")
    @RequestMapping(value = "/comment")
    @ResponseBody
    public CommonResp commetOnTheme(@RequestBody CreateCommentReq req) {
        String userId = "liujm";
        themeManager.comment(req, userId);
        return CommonResp.success();
    }

    @ApiOperation("主题正文")
    @RequestMapping(value = "/main_text")
    @ResponseBody
    public CommonResp getThemeMainText(@RequestParam String topicId) {
        String userId = "liujm";
        MainTextQo mainTextQo = themeManager.getMainText(topicId);
        return CommonResp.success(mainTextQo);
    }

    @ApiOperation("转发主题")
    @RequestMapping(value = "/forward")
    @ResponseBody
    public CommonResp forwardTheme(@Validated @RequestBody ForwardThemeReq forwardThemeReq) {
        String userId = "liujm";
        themeManager.forward(forwardThemeReq, userId);
        return CommonResp.success();
    }


    @ApiOperation("点赞主题")
    @GetMapping(value = "/like")
    @ResponseBody
    public CommonResp likeOnTheme(@RequestParam String themeId) {
        String user = "liujm";
        themeManager.like(themeId, user);
        return CommonResp.success();
    }

    @ApiOperation("分享主题")
    @GetMapping(value = "/share")
    @ResponseBody
    public CommonResp shareTheme(String themeId) {
        return CommonResp.success();
    }

    @ApiOperation("收藏主题")
    @RequestMapping(value = "/book")
    @ResponseBody
    public CommonResp bookTheme(String themeId) {
        String user = "liujm";
//        themeManager.book(themeId,user);
        return CommonResp.success();
    }

    @ApiOperation("举报主题")
    @GetMapping(value = "/complaint")
    @ResponseBody
    public CommonResp complaintTheme(@RequestParam String themeId) {

        return CommonResp.failed("功能暂未开放");
    }

    @ApiOperation("屏蔽")
    @RequestMapping(value = "/conceal")
    @ResponseBody
    public CommonResp concealTheme(String themeId) {
        String user = "liujm";
//        themeManager.blockContent(themeId,user);
        return CommonResp.success();
    }

    @GetMapping(value = "/text")
    @ResponseBody
    public Map<String,String> test(@RequestParam String themeId) {
        HashMap<String, String> result = new HashMap<>();
        result.put("k","v");
        return result;
    }
}
