package com.tanpu.community.manager;

import com.tanpu.common.api.CommonResp;
import com.tanpu.common.exception.BizException;
import com.tanpu.community.api.beans.ForwardThemeReq;
import com.tanpu.community.api.beans.qo.MainTextQo;
import com.tanpu.community.api.beans.qo.ThemeQo;
import com.tanpu.community.api.beans.req.CreateCommentReq;
import com.tanpu.community.api.beans.req.CreateThemeReq;
import com.tanpu.community.api.enums.BlockTypeEnum;
import com.tanpu.community.api.enums.CollectionTypeEnum;
import com.tanpu.community.api.enums.CommentTypeEnum;
import com.tanpu.community.api.enums.ThemeTypeEnum;
import com.tanpu.community.dao.entity.community.*;
import com.tanpu.community.feign.course.FeignClientForCourse;
import com.tanpu.community.feign.course.vo.CourseDetailExt;
import com.tanpu.community.feign.product.FeignClientForProducts;
import com.tanpu.community.feign.tanpuroom.FeignClientForTanpuroom;
import com.tanpu.community.feign.vo.ProductInfoVO;
import com.tanpu.community.feign.vo.ShortVideoBaseInfoResp;
import com.tanpu.community.feign.zhibo.FeignClientForZhibo;
import com.tanpu.community.feign.zhibo.vo.ZhiboDetailVO;
import com.tanpu.community.service.*;
import com.tanpu.community.util.ConvertUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Service
public class ThemeManager {
    @Resource
    private ThemeService themeService;

    @Autowired
    private CollectionService collectionService;

    @Autowired
    private CommentService commentService;

    @Autowired
    private FansRelService fansRelService;

    @Autowired
    private BlackListService blackListService;

    @Autowired
    private FinProResService finProResService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private ZhiboService zhiboService;

    @Autowired
    private CurriculumResService curriculumResService;

    @Autowired
    private OrderFlowService orderFlowService;

    @Autowired
    private ThemeAttachmentService themeAttachmentService;

    @Autowired
    private HomePageService homePageService;

    @Resource
    private FeignClientForTanpuroom feignClientForTanpuroom;

    @Resource
    private FeignClientForZhibo feignClientForZhibo;

    @Resource
    private FeignClientForProducts feignClientForProducts;

    @Autowired
    private FeignClientForCourse feignClientForCourse;

    public void publishTheme(CreateThemeReq req, String userId) {
        ThemeEntity themeEntity = new ThemeEntity();
        BeanUtils.copyProperties(req, themeEntity);
        themeEntity.setAuthorId(userId);

        themeService.insertTheme(themeEntity);
        List<ThemeAttachmentEntity> themeAttachments = ConvertUtil.themeReqToAttachmentList(req, themeEntity.getId());
        //TODO 敏感词过滤
        themeAttachmentService.insertList(themeAttachments);

    }

    // 返回推荐主题文章
    public List<ThemeQo> selectHotThemes(String userId) {
        // TODO:推荐
        List<ThemeEntity> themeEntities = themeService.selectAll();
        List<ThemeQo> themeQos = convertEntityToQo(themeEntities, userId);
        return themeQos;
    }


    // 返回关注主题
    public List<ThemeQo> selectInterestThemes(String userId) {
        List<String> fansList = fansRelService.queryFansByFollowerId(userId);
        List<ThemeEntity> themeEntities = themeService.selectByFans(fansList);
        List<ThemeQo> themeQos = convertEntityToQo(themeEntities, userId);
        return themeQos;
    }

    //计算迄今时间
    private String calUpToNowTime(LocalDateTime start) {
        Duration between = Duration.between(start, LocalDateTime.now());
        long duration = between.toMinutes();
        if (duration < 1) {
            return "刚刚";
        } else if (duration < 60) {
            return duration + "分钟前";
        } else if (duration < 60 * 24) {
            return duration / 60 + "小时前";
        } else if (start.getYear() == LocalDateTime.now().getYear()) {
            return start.format(DateTimeFormatter.ofPattern("MM-dd HH:mm:ss"));
        }
        return start.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }

    // 点赞
    public void like(String themeId, String userId) {
        collectionService.addIfNotExist(themeId, userId, CollectionTypeEnum.LIKE);
    }

    public void unlike(String themeId, String userId) {
    }

    // 评论（对主题）
    public void comment(CreateCommentReq req, String userId) {
        CommentEntity commentEntity = CommentEntity.builder()
                .targetId(req.getThemeId())
                .authorId(userId)
                .createBy(userId)
                .content(req.getComment())
                .commentType(CommentTypeEnum.THEME.getCode())
                .build();

        commentService.insertComment(commentEntity);
    }

    //转发
    public void forward(ForwardThemeReq req, String userId) {
        ThemeEntity targetTheme = themeService.selectById(req.getFormerThemeId());
        ThemeEntity newTheme = ThemeEntity.builder()
                .content(req.getContent())
                .topicId(req.getTopicId())
                .formerThemeId(req.getFormerThemeId())
                .authorId(userId)
                .createBy(userId)
                .themeType(ThemeTypeEnum.DISCUSSION.getCode())
                .build();

        themeService.insertTheme(newTheme);
    }


    //收藏
    public void favorite(String themeId, String userId) {
        collectionService.addIfNotExist(themeId, userId, CollectionTypeEnum.BOOK);
    }

    public void unFavorite(String themeId, String userId) {

    }

    //投诉（主题）
    public void complaint(String themeId, String user) {
        //TODO
    }

    //    屏蔽(用户)
    public void blockUser(String blockUser, String userId) {
        BlackListEntity selectOne = blackListService.selectOne(blockUser, userId, BlockTypeEnum.USER.getCode());
        if (selectOne == null) {
            blackListService.addBlock(blockUser, userId, BlockTypeEnum.USER);
        }
    }

    // 解除屏蔽(用户)
    public void unblockUser(String blockUser, String userId) {
    }

    // 屏蔽(主题)
    public void blockTheme(String themeId, String userId) {
        blackListService.addBlock(themeId, userId, BlockTypeEnum.THEME);
        BlackListEntity selectOne = blackListService.selectOne(themeService.selectById(themeId).getAuthorId(), userId, BlockTypeEnum.USER.getCode());
        if (selectOne == null) {
            blackListService.addBlock(themeService.selectById(themeId).getAuthorId(), userId, BlockTypeEnum.USER);
        }
    }

    // 解除屏蔽(主题)
    public void unblockTheme(String themeId, String userId) {
        blackListService.removeBlackList(themeId, userId, BlockTypeEnum.USER);
    }


    //主题Entity转QO
    private List<ThemeQo> convertEntityToQo(List<ThemeEntity> themeEntities, String userId) {
        List<ThemeQo> themeQos = ConvertUtil.themeEntitiesToDTOs(themeEntities);
        for (ThemeQo themeQO : themeQos) {
            //附件列表
            List<ThemeAttachmentEntity> attachments = themeAttachmentService.selectByThemeId(themeQO.getId());
            if (!CollectionUtils.isEmpty(attachments)) {
                HashMap<String,Object> attachmentQos= new HashMap<>();
                attachments.forEach(o->attachmentQos.put(o.getAttchId(),transferAttachment(o,userId)));
                themeQO.setAttachment(attachmentQos);
            }
            //迄今时间
            themeQO.setUpToNowTime(calUpToNowTime(themeQO.getCreateTime()));
            //是否关注作者
            String authorId = themeQO.getAuthorId();
            Set<String> fansSet = new HashSet<>(fansRelService.queryFansByFollowerId(userId));
            themeQO.setFollow(fansSet.contains(authorId));
            //用户信息：头像、昵称
            HomePageEntity userEntity = homePageService.selectByUserId(userId);
            themeQO.setUserImg(userEntity.getHeadImg());
            themeQO.setNickName(userEntity.getNickName());


        }
        return themeQos;
    }


    public Object transferAttachment(ThemeAttachmentEntity themeAttachment, String userId) {
        String attachmentId = themeAttachment.getAttchId();
        switch (themeAttachment.getAttachType()) {
            //附件类型 1:产品 2：直播 3：短视频 4：课程
            //TODO ENTITY 转 DTO
            case 1:
                CommonResp<ProductInfoVO> productDetailInfo = feignClientForProducts.getProductDetailInfo(attachmentId);
                return productDetailInfo.getData();
//                return finProResService.selectById(themeAttachment.getAttchId());

            case 2:
                CommonResp<ZhiboDetailVO> zhiboDetailVOCommonResp = feignClientForZhibo.queryZhiboDetail(attachmentId);
                return zhiboDetailVOCommonResp.getData();
//                return zhiboService.selectByid(themeAttachment.getAttchId());
            case 3:
                String[] attachmentIds = new String[]{attachmentId};
                CommonResp<List<ShortVideoBaseInfoResp>> shortVideoBaseInfo =
                        feignClientForTanpuroom.getShortVideoBaseInfo(Arrays.asList(attachmentIds));
                return shortVideoBaseInfo.getData().get(0);
//                CurriculumResEntity curriculumResEntity = curriculumResService.selectShortVideo(themeAttachment.getAttchId());
//                return curriculumResEntity;
            case 4:
                String[] split = attachmentId.split(",");
                if (split.length != 2) {
                    throw new BizException("课程id格式错误：" + attachmentId);
                }
                String courseId = split[0];
                String packageId = split[1];
                CommonResp<CourseDetailExt> courseIdDetailInfo = feignClientForCourse.getCourseIdDetailInfo(courseId, packageId);
                return courseIdDetailInfo.getData();

//                CurriCulumInfoDetailDto curriCulumInfoDetailDto = CurriCulumInfoDetailDto.builder().curriculumId(attachmentId).userId(userId).build();
//                CommonResp<CurriculumDetailVo> curriculumDetailVoCommonResp = feignClientForTanpuroom.queryCurriculumDetail(curriCulumInfoDetailDto);
//                return curriculumDetailVoCommonResp.getData();
//                curriculumResEntity = curriculumResService.selectById(themeAttachment.getAttchId());
//
//                OrderFlowEntity orderFlowEntity = orderFlowService.queryByUserAndProId(themeAttachment.getThemeId(), userId);
//                return curriculumResEntity;
            default:
                return null;


        }
    }


    public MainTextQo getMainText(String topicId) {
        return null;
    }
}
